﻿/****************************************************************/
/*  HETN-DIO864T/HWIF-DIO864W Sample Program(Visual C/C++)		*/	
/*	    file name:sed86400.c									*/
/*      OS       :Windows										*/
/*      date     :2019/07/11									*/
/*	Copyright(C) 2018-2019 Hivertec Inc. All Rights Reserved.	*/
/****************************************************************/
#include	<windows.h>
#include	<stdio.h>
#include	"hedio864.h"
#include	"ed864l1a.h"
#include	"resource.h"

#define		IO_MAX			32		// 入力/出力ポート　ボタン他 最大数
#define		CNTMAX			16		// デバイス最大個数(HETN/HWIF合わせて16枚まで)
#define		IDM_TMR_INOUT	11021	// 入力/出力ポート表示用タイマーコントロールID

HBITMAP		hBitGreen, hBitWhite;	// ビットマップ
HWND		hBtnOut  [IO_MAX];		// ボタン(出力ポート)
HWND		hStaIn   [IO_MAX];		// スタティック(入力ポート)
HWND		hStaOutNo[IO_MAX];		// スタティック(出力ポート ビットNO)
HWND		hStaInNo [IO_MAX];		// スタティック(入力ポート ビットNO)
HINSTANCE	hInst;					// インスタンスハンドル
HETNDEVINF	hInf[CNTMAX] = {0};		// デバイス情報(全体)
HETNDEVINF	gInf = {0};				// デバイス情報(オープン)
int			gDevNo = 0;				// デバイスインデックス番号
short		gDevTyp = 0;			// デバイスタイプ
short		gBrdId = 0;				// ボードID
DWORD		gFilter = 0;			// フィルター値

// 関数定義
LRESULT	CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
void	FormLoad(HWND);					// プログラムロード
void	FormUnload(HWND);				// プログラム終了処理
void	CmbInfo_Click(HWND, WPARAM);	// デバイス情報選択コンボボックス
void	BtnInfo_Click(HWND);			// デバイス情報取得
void	BtnOpen_Click(HWND);			// デバイスオープン
void	BtnClose_Click(HWND);			// デバイスクローズ
void	BtnOut_Click (HWND, WORD);		// 出力ポート(ON/OFF)
void	BtnFlt_Click (HWND);			// フィルター設定
void	TmrInOut(HWND);					// 入力/出力ポート表示用タイマー
void	ErrMsg(HWND, DWORD);			// エラーメッセージ表示処理

// DIO464U/864T/864Wボードフィルター値テーブル
DWORD	newfltvaltbl[16] = {
		0,			// [0] 0us(なし)
		5L,			// [1] 5us
		10L,		// [2] 10us
		20L,		// [3] 20us
		40L,		// [4] 40us
		80L,		// [5] 80us
		100L,		// [6] 100us
		200L,		// [7] 200us
		400L,		// [8] 400us
		800L,		// [9] 800us
		1000L,		// [10] 1ms
		4000L,		// [11] 4ms
		8000L,		// [12] 8ms
		16000L,		// [13] 16ms
		32000L,		// [14] 32ms
		64000L		// [15] 64ms
	};

int APIENTRY WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst, LPSTR lqszArgs, int nWinMode)
{
	HWND		hwnd;
	MSG			msg;
	WNDCLASSEX	wc;
	char        szAppName[] = "MainDlg", *dummy;
	
	// 警告を減らすためのダミー処理
	dummy = lqszArgs;	

	if(!hPrevInst)	{
		// ウィンドウクラスを定義する
		wc.style		 =	CS_HREDRAW | CS_VREDRAW;			// ウィンドウの種類(サイズ変更時再描画)
		wc.lpfnWndProc	 =	(WNDPROC)WndProc;					// ウインドウ関数のアドレス
		wc.cbClsExtra	 =	0;									// その他のクラス情報
		wc.cbWndExtra	 =	DLGWINDOWEXTRA;						// その他のウィンドウ情報
		wc.hInstance	 =	hThisInst;							// このインスタンスのハンドル
		wc.hIcon		 =	LoadIcon(NULL, IDI_APPLICATION);	// 標準アイコンのハンドル
		wc.hCursor		 =	LoadCursor(NULL, IDC_ARROW);		// マウスカーソルのハンドル
		wc.hbrBackground =	(HBRUSH)(COLOR_BTNFACE + 1);		// ウィンドウの背景色(ボタンの表面の影)
		wc.lpszMenuName	 =	NULL;								// メインメニューの名前
		wc.lpszClassName =	szAppName;							// ウィンドウクラスの名前
		wc.cbSize		 =	sizeof(WNDCLASSEX);					// WNDCLASSEX構造体のサイズを設定
		wc.hIconSm		 =	LoadIcon(NULL, IDI_WINLOGO);		// スモールアイコン
		
		// ウィンドウクラスを登録する
		if(!RegisterClassEx(&wc)) return 0;
	}

	// 現在のインスタンスハンドルを保存
	hInst = hThisInst;

	// ウィンドウクラスの登録ができたので、メインのダイアログボックスを生成する
 	hwnd = CreateDialog(hInst, "IDD_DIALOG0", 0, (DLGPROC)WndProc);
	if(!hwnd)		return 0;

	// ウィンドウを表示する
	ShowWindow  (hwnd, nWinMode);
	UpdateWindow(hwnd);

	// メッセージループを生成する
	while (GetMessage(&msg, NULL, 0, 0)) {
		if(!IsDialogMessage(hwnd, &msg)) {
			TranslateMessage(&msg);						// キーボード利用を可能にする
			DispatchMessage (&msg);						// 制御をWindowsに戻す
		}
	}
	return (int)msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		// プログラム開始処理
		case WM_INITDIALOG:	FormLoad(hwnd);							break;

		// ボタンクリック時の処理
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				// デバイス情報取得
				case IDC_BTN_INFO:		BtnInfo_Click(hwnd);			break;
				
				// デバイス情報選択 & 表示
				case IDC_CMB_INFO:		CmbInfo_Click(hwnd, wParam);	break;

				// デバイスオープン
				case IDC_BTN_OPEN:		BtnOpen_Click(hwnd);			break;

				// デバイスクローズ
				case IDC_BTN_CLOS:		BtnClose_Click(hwnd);			break;

				// 出力ポート(ON/OFF)ボタン
				case IDC_BTN_OUT1:		BtnOut_Click(hwnd, 0);			break;
				case IDC_BTN_OUT2:		BtnOut_Click(hwnd, 1);			break;
				case IDC_BTN_OUT3:		BtnOut_Click(hwnd, 2);			break;
				case IDC_BTN_OUT4:		BtnOut_Click(hwnd, 3);			break;
				case IDC_BTN_OUT5:		BtnOut_Click(hwnd, 4);			break;
				case IDC_BTN_OUT6:		BtnOut_Click(hwnd, 5);			break;
				case IDC_BTN_OUT7:		BtnOut_Click(hwnd, 6);			break;
				case IDC_BTN_OUT8:		BtnOut_Click(hwnd, 7);			break;
				case IDC_BTN_OUT9:		BtnOut_Click(hwnd, 8);			break;
				case IDC_BTN_OUT10:		BtnOut_Click(hwnd, 9);			break;
				case IDC_BTN_OUT11:		BtnOut_Click(hwnd, 10);			break;
				case IDC_BTN_OUT12:		BtnOut_Click(hwnd, 11);			break;
				case IDC_BTN_OUT13:		BtnOut_Click(hwnd, 12);			break;
				case IDC_BTN_OUT14:		BtnOut_Click(hwnd, 13);			break;
				case IDC_BTN_OUT15:		BtnOut_Click(hwnd, 14);			break;
				case IDC_BTN_OUT16:		BtnOut_Click(hwnd, 15);			break;
				case IDC_BTN_OUT17:		BtnOut_Click(hwnd, 16);			break;
				case IDC_BTN_OUT18:		BtnOut_Click(hwnd, 17);			break;
				case IDC_BTN_OUT19:		BtnOut_Click(hwnd, 18);			break;
				case IDC_BTN_OUT20:		BtnOut_Click(hwnd, 19);			break;
				case IDC_BTN_OUT21:		BtnOut_Click(hwnd, 20);			break;
				case IDC_BTN_OUT22:		BtnOut_Click(hwnd, 21);			break;
				case IDC_BTN_OUT23:		BtnOut_Click(hwnd, 22);			break;
				case IDC_BTN_OUT24:		BtnOut_Click(hwnd, 23);			break;
				case IDC_BTN_OUT25:		BtnOut_Click(hwnd, 24);			break;
				case IDC_BTN_OUT26:		BtnOut_Click(hwnd, 25);			break;
				case IDC_BTN_OUT27:		BtnOut_Click(hwnd, 26);			break;
				case IDC_BTN_OUT28:		BtnOut_Click(hwnd, 27);			break;
				case IDC_BTN_OUT29:		BtnOut_Click(hwnd, 28);			break;
				case IDC_BTN_OUT30:		BtnOut_Click(hwnd, 29);			break;
				case IDC_BTN_OUT31:		BtnOut_Click(hwnd, 30);			break;
				case IDC_BTN_OUT32:		BtnOut_Click(hwnd, 31);			break;

				// フィルター設定ボタン
				case IDC_BTN_SETFLT:	BtnFlt_Click(hwnd);				break;
			}
			break;

		// タイマ処理
		case WM_TIMER:
			switch(LOWORD(wParam)) {
				// 入力/出力ポート表示
				case IDM_TMR_INOUT:	TmrInOut(hwnd);					break;
			}
																	break;
		// ウィンドウを閉じる時の処理
		case WM_CLOSE:		FormUnload(hwnd);						break;

		// プログラム終了処理
		case WM_DESTROY:	PostQuitMessage(0);						break;

		// 上記以外のメッセージをWindows に処理させる
		default:	return DefWindowProc(hwnd, message, wParam, lParam);
	}
	return 0;
}

// フォームロード
void FormLoad(HWND hwnd)
{
	RECT	rectW;
	WORD	wIx;
	int		nX, nY;

	// 画面座標を取得する
	nX = GetSystemMetrics(SM_CXSCREEN);		// 画面全体の幅
	nY = GetSystemMetrics(SM_CYSCREEN);		// 画面全体の高さ

	// ウィンドウの境界矩形の寸法を取得する。
	GetWindowRect(hwnd, &rectW);
	rectW.right  = rectW.right  - rectW.left;
	rectW.bottom = rectW.bottom - rectW.top;

	// ウィンドウを中央に移動する(高さは中央より少し上)
	rectW.top  = (nY - rectW.bottom) / 2 - 15;
	rectW.left = (nX - rectW.right ) / 2;
	MoveWindow(hwnd, rectW.left, rectW.top, rectW.right, rectW.bottom, TRUE);
	
	// ボタン使用可/不可とイメージ表示のセット
	EnableWindow(GetDlgItem(hwnd, IDC_CMB_INFO), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_OPEN), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_CLOS), FALSE);

	// リソースファイルで作成した入力/出力ポートのボタンとスタティックのハンドルを取得する｡
	for(wIx = 0; wIx < IO_MAX; wIx++)	{
		hBtnOut  [wIx] = GetDlgItem(hwnd, IDC_BTN_OUT1   + wIx);
		hStaOutNo[wIx] = GetDlgItem(hwnd, IDC_STA_OUTNO1 + wIx);

		hStaIn   [wIx] = GetDlgItem(hwnd, IDC_STA_IN1   + wIx);
		hStaInNo [wIx] = GetDlgItem(hwnd, IDC_STA_INNO1 + wIx);
	}

	// ビットマップのロード
	hBitGreen = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_GREEN));
	hBitWhite = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_WHITE));
}

// フォームアンロード
void FormUnload(HWND hwnd)
{
	/* デバイスクローズ */
	ed864_CloseDevice(gDevTyp, gBrdId);

	// デバイスコンテキストを削除する
	DeleteObject(hBitGreen);
	DeleteObject(hBitWhite);

	// メインのダイアログボックスを閉じる
	DestroyWindow(hwnd);
}

// デバイス情報取得
void BtnInfo_Click(HWND hwnd)
{
	char	buf[256];
	char	tmp[64];
	WORD	wIx;
	WORD	id;
	WORD	pver;	// HW version
	WORD	lver;	// SW version
	DWORD	ret;
	DWORD	cnt;	// デバイス個数

    // カーソルを砂時計（情報取得に時間がかかるため）
	HCURSOR cur = SetCursor(LoadCursor(NULL,IDC_WAIT));

	// 常駐プログラムリンク設定(自動検出)
	ret = hed864_ProgLinkSet(0, "", 0);
	if (ret)
	{
		SetCursor(cur);//カーソルを元に戻す
		wsprintf( buf, "Regident program not running.\nPlease make sure the device is turned on and \n\'Hivertec Windows Socket Program\' running.\nReturn:%04Xh", ret );
		MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
		return;
	}

	// 常駐プログラムバージョン取得と表示
	ret = hed864_GetProgVersion(&pver, &lver);
	if (ret)
	{
		SetCursor(cur);//カーソルを元に戻す
		wsprintf( buf, "Cannot get regident program versions.\nReturn:%04Xh", ret );
		MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
		return;
	}
	wsprintf(buf,"%s%s%01X.%03X%s%01X.%03X",	
		"*** Regident Program Version *** \n",
		  " Program: ",pver/0x1000,pver%0x1000,
		"\n Library: ",lver/0x1000,lver%0x1000);
	SetDlgItemText(hwnd, IDC_STA_RPVER, buf);

    // カーソルを元に戻す
	SetCursor(cur);//カーソルを元に戻す

	// ボード枚数取得
    ret = ed864_GetDeviceCount(&cnt);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}

    // ボードが 0枚の時エラー
    if(!cnt) {
		MessageBox(hwnd, "HUSB-DIO464 seriese device not found.", "sed86400", MB_OK | MB_ICONERROR);
		return;
	}
	
    // ボードが17枚以上の時エラー
    if(cnt > CNTMAX) {
		MessageBox(hwnd, "HUSB-DIO464 seriese devices too mach.(over 16 devices) ", "sed86400", MB_OK | MB_ICONERROR);
		return;
	}

	// デバイス情報取得
	ret = ed864_GetDeviceInfo(&cnt, &hInf[0]);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}

	// デバイス情報コンボボックスの内容をリセットする
	SendMessage(GetDlgItem(hwnd, IDC_CMB_INFO), CB_RESETCONTENT, 0l, 0l);

	// デバイス情報コンボボックスにテキストアイテムを追加する
	for (wIx = 0; wIx < cnt; wIx++) {
		// コンボボックスにテキストアイテムを追加する
        id = hInf[wIx].DevTyp;
	    id <<= 4;
		id += hInf[wIx].BrdID;
	    if(hInf[wIx].DevTyp == 0x12) strcpy_s(tmp, sizeof(tmp), "HETN");
		else if(hInf[wIx].DevTyp == 0x13) strcpy_s(tmp, sizeof(tmp), "HWIF");
		else strcpy_s(tmp, sizeof(tmp), "----");

		wsprintf(buf, "%02X  Type=%s Bid=%X", id, tmp, hInf[wIx].BrdID);
		SendMessage(GetDlgItem(hwnd, IDC_CMB_INFO), CB_ADDSTRING, 0L, (LPARAM)buf);
	}

    // デバイスオープン条件のデフォルト設定(デバイス番号、1枚め)
	SendMessage(GetDlgItem(hwnd, IDC_CMB_INFO), CB_SETCURSEL, 0l, 0l);

	// ボタン使用可/不可とボタン・スタティック表示のセット
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_OPEN), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMB_INFO), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_CLOS), FALSE);

	CmbInfo_Click(hwnd, MAKEWPARAM(IDC_CMB_INFO, CBN_SELCHANGE));
}

// デバイス選択
void CmbInfo_Click(HWND hwnd, WPARAM wParam)
{

}

// デバイスオープン
void BtnOpen_Click(HWND hwnd)
{
	WORD	wIx;
	DWORD	ret;
	WORD	ver[2];
	char	buf[256];
	char	tmp[64];
	short	num;
	int		ans;

    // 選択されたデバイスが何枚目か取得
	wIx = (WORD)SendMessage(GetDlgItem(hwnd, IDC_CMB_INFO), CB_GETCURSEL, 0l, 0l);
	if(wIx < 0) {
		ErrMsg(hwnd, ILLEGAL_NETPARAM);
		return;
	}

	// 選択デバイス情報保存
	gDevNo = wIx;
	gInf = hInf[gDevNo];
	gDevTyp = gInf.DevTyp; 
	gBrdId = gInf.BrdID;

	//デバイスオープン済みかチェック
	ret = ed864_GetOpenDevCount(gDevTyp, gBrdId, &num);
	if(ret) {
		ErrMsg(hwnd, ret);
		return;
	}
	//既にオープンしている場合はオープンして良いか確認する
	if(num != 0) {
		ans = MessageBox( hwnd, "既にオープンされているデバイスです．オープンしますか？", "複数起動", MB_YESNO );
		if( ans != IDYES ) {
			return;
		}
	}

	// デバイスオープン
	ret = ed864_OpenDevice(gDevTyp, gBrdId);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}

	// デバイス情報表示
	strcpy_s(buf, sizeof(buf), "*** Device Information ***\n");
	if(gDevTyp == 0x12)			strcpy_s(tmp, sizeof(tmp), "Board Type  : HETN-DIO834T\n");
    else if(gDevTyp == 0x13)	strcpy_s(tmp, sizeof(tmp), "Board Type  : HWIF-DIO834W\n");
    else						strcpy_s(tmp, sizeof(tmp), "Board Type  : ------------\n");
	strcat_s(buf, sizeof(buf), tmp);
	wsprintf(tmp, "Board ID    : 0x%X\n", gBrdId);
	strcat_s(buf, sizeof(buf), tmp);
	wsprintf(tmp, "IP Address  : %d.%d.%d.%d\n",
			gInf.IpAdr[0], gInf.IpAdr[1],
			gInf.IpAdr[2], gInf.IpAdr[3]);
	strcat_s(buf, sizeof(buf), tmp);
	wsprintf(tmp, "MAC Address : %02X%02X%02X%02X%02X%02X\n",
			gInf.MacAdr[0], gInf.MacAdr[1],
			gInf.MacAdr[2], gInf.MacAdr[3],
			gInf.MacAdr[4], gInf.MacAdr[5]);
	strcat_s(buf, sizeof(buf), tmp);
	SetDlgItemText(hwnd, IDC_STA_INFO, buf);

	SetFocus(GetDlgItem(hwnd, IDC_CMB_INFO));

	// デバイスバージョン情報取得
	ret = ed864_rVersion(gDevTyp, gBrdId, ver);
	if(ret) {
		ErrMsg(hwnd, ret);
		return;
	}
	wsprintf(buf,"%s%s%01X.%03X%s%01X.%03X",	
		"*** Device Version*** \n",
		  " Hardware: ",ver[0]/0x1000,ver[0]%0x1000,
		"\n Firmware: ",ver[1]/0x1000,ver[1]%0x1000);

	SetDlgItemText(hwnd, IDC_STA_VERS, buf);

	// ボタン使用可/不可とイメージ表示のセット
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_INFO), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_OPEN), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMB_INFO), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_CLOS), TRUE);

	for(wIx = 0; wIx < IO_MAX; wIx++) {
		EnableWindow(hBtnOut[wIx], TRUE);
		ShowWindow(hBtnOut  [wIx], SW_SHOW);
		ShowWindow(hStaOutNo[wIx], SW_SHOW);
		ShowWindow(hStaInNo[wIx], SW_SHOW);
	}

	// フィルターコンボボックスの内容をリセットする
	SendMessage(GetDlgItem(hwnd, IDC_CMB_FILTER), CB_RESETCONTENT, 0l, 0l);

	// フィルターコンボボックスにテキストアイテムを追加する
	for(wIx = 0; wIx < 16; wIx++) {
		wsprintf(buf, "%d (%d.%03dms)", wIx, newfltvaltbl[wIx]/1000, newfltvaltbl[wIx]%1000);
		SendMessage(GetDlgItem(hwnd, IDC_CMB_FILTER), CB_ADDSTRING, 0L, (LPARAM)buf);
	}
	SendMessage(GetDlgItem(hwnd, IDC_CMB_FILTER), CB_SETITEMHEIGHT, 0, 0);

	EnableWindow(GetDlgItem(hwnd, IDC_CMB_FILTER), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_BTN_SETFLT), TRUE);

	//フィルター値を全ﾎﾟｰﾄ/全ﾋﾞｯﾄを1msに設定(U/T/W専用関数を使用)
	gFilter = 10;	//filter no=10(1ms)
	ret = ed864_SetFilter(gDevTyp, gBrdId, 0xff, (WORD)gFilter);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}
	wsprintf(buf, "%d.%d", newfltvaltbl[gFilter]/1000, newfltvaltbl[gFilter]%1000);
	SetDlgItemText(hwnd, IDC_STA_FLTVAL, buf);

	// 入力/出力ポート表示用タイマー開始(110msec)
	SetTimer(hwnd, IDM_TMR_INOUT, 110, NULL);

	// フォーカスを移動する
	SetFocus(hwnd);
}

// デバイスクローズ
void BtnClose_Click(HWND hwnd)
{
    WORD	wIx;
	DWORD	ret;

	if(gDevTyp != 0) {
		// 入力/出力ポート表示用タイマー終了
		KillTimer(hwnd, IDM_TMR_INOUT);

		// 入力/出力ポート表示クリア
		for(wIx = 0; wIx < IO_MAX; wIx++) {
			SendDlgItemMessage(hwnd, IDC_STA_IN1  + wIx, STM_SETIMAGE,
							   (WPARAM)IMAGE_BITMAP, (LPARAM)NULL);
			SendDlgItemMessage(hwnd, IDC_BTN_OUT1 + wIx, BM_SETIMAGE,
							   (WPARAM)IMAGE_BITMAP, (LPARAM)NULL);
		}

		// すべての出力ポートに０を出力
		ret = ed864_wOutDW(gDevTyp, gBrdId, 0);
		if(ret) {	
			ErrMsg(hwnd, ret);
			return;
		}

		// デバイス情報表示クリア
		SetDlgItemText(hwnd, IDC_STA_INFO, "");
		// デバイスバージョン情報表示クリア
		SetDlgItemText(hwnd, IDC_STA_VERS, "");
		// フィルター表示クリア
		SetDlgItemText(hwnd, IDC_STA_FLTVAL, "");
		SendMessage(GetDlgItem(hwnd, IDC_CMB_FILTER), CB_RESETCONTENT, 0l, 0l);

		// デバイスクローズ
		ret = ed864_CloseDevice(gDevTyp, gBrdId);
		if(ret) {	
			ErrMsg(hwnd, ret);
			return;
		}

        // デバイスクローズ済ならばgDevTypを0クリア
        gDevTyp = 0;

		// ボタン使用可/不可とボタンイメージ非表示のセット
		EnableWindow(GetDlgItem(hwnd, IDC_BTN_INFO), TRUE);
		EnableWindow(GetDlgItem(hwnd, IDC_BTN_OPEN), TRUE);
		EnableWindow(GetDlgItem(hwnd, IDC_CMB_INFO), TRUE);
		EnableWindow(GetDlgItem(hwnd, IDC_BTN_CLOS), FALSE);

		for(wIx = 0; wIx < IO_MAX; wIx++) {
			EnableWindow(hBtnOut[wIx], FALSE);
			ShowWindow(hBtnOut  [wIx], SW_HIDE);
			ShowWindow(hStaOutNo[wIx], SW_HIDE);
			ShowWindow(hStaInNo [wIx], SW_HIDE);
		}

		EnableWindow(GetDlgItem(hwnd, IDC_CMB_FILTER), FALSE);
		EnableWindow(GetDlgItem(hwnd, IDC_BTN_SETFLT), FALSE);

		// フォーカスを移動する
		SetFocus(GetDlgItem(hwnd, IDC_CMB_INFO));
	}
}

// タイマー周期処理（入出力ポート状態読込）
void TmrInOut(HWND hwnd)
{
	WORD	nBit;
	DWORD	rdt;
	DWORD	ret;

	if(gDevTyp != 0) {
		// 入力ポート読込
		ret = ed864_rInDW(gDevTyp, gBrdId, &rdt);
		if(ret) {	
			KillTimer(hwnd, IDM_TMR_INOUT);
			ErrMsg(hwnd, ret);
			return;
		}

		// 入力ポート結果表示
		for(nBit = 0; nBit < 32; nBit++) {
			if(rdt & (0x01 << nBit)) {
				// 入力ポート ON  ; 緑色表示
				SendDlgItemMessage(hwnd, IDC_STA_IN1 + nBit, STM_SETIMAGE, 
								   (WPARAM)IMAGE_BITMAP, (LPARAM)hBitGreen);
			} else {
				// 入力ポート OFF ; 白色表示
				SendDlgItemMessage(hwnd, IDC_STA_IN1 + nBit, STM_SETIMAGE, 
								   (WPARAM)IMAGE_BITMAP, (LPARAM)hBitWhite);
			}
		}

		// 出力ポート読込
		ret = ed864_rOutDW(gDevTyp, gBrdId, &rdt);
		if(ret) {	
			KillTimer(hwnd, IDM_TMR_INOUT);
			ErrMsg(hwnd, ret);
			return;
		}

		// 出力ポート結果表示
		for(nBit = 0; nBit < 32; nBit++) {
			if(rdt & (0x01 << nBit)) {
				// 出力ポート ON  ; 緑色表示
				SendDlgItemMessage(hwnd, IDC_BTN_OUT1 + nBit, BM_SETIMAGE,
								   (WPARAM)IMAGE_BITMAP, (LPARAM)hBitGreen);
			} else {
				// 出力ポート OFF ; ボタンにビットマップを貼り付けない。
				SendDlgItemMessage(hwnd, IDC_BTN_OUT1 + nBit, BM_SETIMAGE,
									   (WPARAM)IMAGE_BITMAP, (LPARAM)NULL);
			}
		}
	}
}

// 出力ポート書込
void BtnOut_Click(HWND hwnd, WORD wNo)
{
	DWORD	rdt;
	DWORD	wdt;
	DWORD	ret;

	// 出力ポート読込
	ret = ed864_rOutDW(gDevTyp, gBrdId, &rdt);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}

    // 出力ポートの該当ビットのON/OFF切替を行う
	wdt = 0x01 << wNo;
	wdt = wdt ^ rdt;

	// 出力ポート書込
	ret = ed864_wOutDW(gDevTyp, gBrdId, wdt);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}
}

// フィルター設定書込
void BtnFlt_Click(HWND hwnd)
{
	char	buf[256];
	int		idx;
	DWORD	dat;
	DWORD	ret;

	idx = (int)SendMessage(GetDlgItem(hwnd, IDC_CMB_FILTER), CB_GETCURSEL, 0, 0);
	if(idx == -1) {    // 未選択チェック
		MessageBox(hwnd, "Data not select.", "sed86400", MB_OK | MB_ICONERROR);
		return;
	}
	//フィルター値を全ﾎﾟｰﾄに書込み
	dat = newfltvaltbl[idx];
	ret = ed864_SetFilter(gDevTyp, gBrdId, 0xff, (WORD)idx);
	if(ret) {	
		ErrMsg(hwnd, ret);
		return;
	}
	gFilter = idx;
	wsprintf(buf, "%d.%03d", dat/1000, dat%1000);
	SetDlgItemText(hwnd, IDC_STA_FLTVAL, buf);
}

// エラーメッセージ
void ErrMsg(HWND hwnd, DWORD dwRet)
{
	char	buf[256];

	switch(dwRet) {
		case INVALID_SOCHDL:
			wsprintf(buf, "Invalid socket handle.\nReturn value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case NETDEV_NOTFOUND:
			wsprintf(buf, "Device not found.\nReturn value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case RECEIVE_TIMEOUT:
			wsprintf(buf, "ethernet/WiFi communication timeout.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case BUFFER_OVERFLOW:
			wsprintf(buf, "Receive data buffer overflow.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_NETPARAM:
			wsprintf(buf, "Parameter over range.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_FORMAT:
			wsprintf(buf, "Invalid data format.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_RESPONSE:
			wsprintf(buf, "Invalid response data received.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_NETDEVICE:
			wsprintf(buf, "Target device not detect.\nReturn value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_NETACCESS:
			wsprintf(buf, "Access denied.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		case ILLEGAL_REGPROG:
			wsprintf(buf, "Regident program invalid or not started.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
		default:
			wsprintf(buf, "Other error.\nReturn Value:%#lx", dwRet);
			MessageBox(hwnd, buf, "sed86400", MB_OK | MB_ICONERROR);
			break;
	}
}
